/*
    WPX - WordPad hook
    Copyright (c) 1995-2003 Alessandro Scotti
    http://www.walkofmind.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#define STRICT
#include <windows.h>
#include <windowsx.h>
#include "wpx.h"

/* Global variables */
static CHAR         szAppClass[] = WPXCLASSNAME;
static HINSTANCE    hAppInst;
static CHAR         szCmdLine[256];

/* Displays an error box and closes the specified window */
LRESULT ErrorBox( HWND hWnd, LPSTR pszMsg )
{
    MessageBox( hWnd,
        pszMsg,
        "Error",
        MB_OK | MB_ICONEXCLAMATION );
    if( hWnd != NULL )
        DestroyWindow( hWnd );
    return( TRUE );
}

/* Enumerates all WordPad windows */
BOOL CALLBACK subEnumSubclass( HWND hWnd, LPARAM lUser )
{
    CHAR    szBuf[128];

    if( GetClassName( hWnd, szBuf, sizeof(szBuf) ) > 0 ) {
        if( lstrcmp( szBuf, WORDPADCLASSNAME ) == 0 ) {
            /* Subclass this window */
            PostMessage( hWnd, lUser, (WPARAM)hWnd, 0 );
        }
    }
    return( TRUE );
}

/* Main window procedure */
LRESULT CALLBACK WndProc( HWND hWnd, UINT wMsg, WPARAM wParam, LPARAM lParam )
{
    HWND            hWndTmp;
    static DWORD    dwWindowCount = 0;
    static HHOOK    hShlHook = NULL;
    static HHOOK    hMsgHook = NULL;

    switch( wMsg ) {
        /* Window creation */
        case WM_CREATE:
            /* Install shell hook */
            if( (hShlHook = wpxInitShellHook( 0 )) == NULL ) {
                ErrorBox( hWnd, "Cannot install a shell hook" );
                break;
            }
            /* Find WordPad window */
            hWndTmp = FindWindow( WORDPADCLASSNAME, NULL );
            if( hWndTmp == NULL ) {
                /* Window does not exist, run WordPad */
                HINSTANCE   hInst;

                /* Execute */
                hInst = ShellExecute( NULL,
                    NULL,
                    "WORDPAD.EXE",
                    szCmdLine,
                    "",
                    SW_SHOWNORMAL );
                if( (DWORD)hInst <= 32 )
                    ErrorBox( hWnd, "Cannot execute WordPad" );
            }
            else {
                /* One or more windows already exist, use message hook */
                hMsgHook = wpxInitMsgHook( 0 );
                EnumWindows( subEnumSubclass, PM_SUBCLASSWINDOW );
            }
            break;
        /* Another window has been subclassed, keep count */
        case PM_INSERTWINDOW:
            dwWindowCount++;
            PostMessage( (HWND)wParam, PM_INITGUI, 0, 0 );
            return( 0 );
        /* Return hook information */
        case PM_GETMSGHOOK:
            return( (LRESULT)hMsgHook );
        case PM_GETSHELLHOOK:
            return( (LRESULT)hShlHook );
        /* A subclassed window is closing or unsubclassing */
        case PM_REMOVEWINDOW:
            if( dwWindowCount > 0 ) {
                dwWindowCount--;
                /* Close if no more windows */
                if( dwWindowCount == 0 )
                    DestroyWindow( hWnd );
            }
            return( 0 );
        /* Close request */
        case WM_CLOSE:
            if( dwWindowCount > 0 ) {
                EnumWindows( subEnumSubclass, PM_UNSUBCLASSWINDOW );
                return( 0 );
            }
            break;
        /* Quit application */
        case WM_DESTROY:
            wpxTermMsgHook();
            wpxTermShellHook();
            PostQuitMessage( 0 );
            return( 0 );
    }
    return( DefWindowProc( hWnd, wMsg, wParam, lParam ) );
}

/* Program entry point */
int PASCAL WinMain( HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR lpszCmd, int nCmdShow )
{
    HWND        hWnd;
    MSG         stMsg;
    WNDCLASS    stWndClass;
    BOOL        boRes;
    HANDLE      hMtxInst;
    DWORD       dwRes;

    NOREF(hPrevInst);
    NOREF(nCmdShow);

    /* Avoid multiple instances */
    hMtxInst = CreateMutex( NULL, FALSE, WPXMUTEXNAME );
    if( hMtxInst == NULL )
        return( 0 );
    dwRes = WaitForSingleObject( hMtxInst, 0 );
    if( (dwRes != WAIT_OBJECT_0) && (dwRes != WAIT_ABANDONED) )
        return( 0 );

    /* Initialize global variables */
    hAppInst = hInst;               /* Save instance handle */
    lstrcpy( szCmdLine, lpszCmd );  /* Save command line (for WordPad) */

    /* Set return value to zero */
    stMsg.wParam = 0;

    /* Register window class */
    stWndClass.style            = CS_HREDRAW | CS_VREDRAW;
    stWndClass.lpfnWndProc      = (WNDPROC)WndProc;
    stWndClass.cbClsExtra       = 0;
    stWndClass.cbWndExtra       = 0;
    stWndClass.hInstance        = hInst;
    stWndClass.hIcon            = (HICON)NULL;
    stWndClass.hCursor          = (HCURSOR)NULL;
    stWndClass.hbrBackground    = (HBRUSH)NULL;
    stWndClass.lpszMenuName     = NULL;
    stWndClass.lpszClassName    = szAppClass;
    if( RegisterClass( &stWndClass ) == 0 )
        goto lblExit;

    /* Create hidden window */
    hWnd = CreateWindow( szAppClass,
        WPXWINDOWNAME,
        WS_POPUP,
        0, 0,
        0, 0,
        NULL,
        NULL,
        hInst,
        NULL );
    if( hWnd == NULL )
        goto lblExit;

    /* Enter main loop */
    while( GetMessage( &stMsg, NULL, 0, 0 ) ) {
        TranslateMessage( &stMsg );
        DispatchMessage( &stMsg );
    }

lblExit:
    ReleaseMutex( hMtxInst );
    return( stMsg.wParam );
}

